﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ConsoleApp7
{
    /// <summary>
    /// Egy lehetséges felállás, amivel a válogatottat ki lehet állítani
    /// </summary>
    public class Config
    {
        /// <summary>
        /// A kiválaszott játékosok
        /// </summary>
        public List<Player> Players { get; set; }

        /// <summary>
        /// A kiválaszott játékosok pontszámai
        /// </summary>
        public int Points { get; set; }

        /// <summary>
        /// Létrehoz egy lehetséges felállást a megadott konfigurációnak megfelelően
        /// </summary>
        /// <param name="defender">Védők száma</param>
        /// <param name="midfielder">Középpályások száma</param>
        /// <param name="forward">Csatárok száma</param>
        /// <param name="points">Játékosonként pontokat tartalmazó szótár</param>
        public Config(int defender, int midfielder, int forward, Dictionary<Player,int> points)
        {
            var configParts = points.GroupBy(p => p.Key.Position, // Csoportosítjuk a szótár elemeit posztonként
              (pos, posPlayers) => // pos: a kulcs, posPlayers: a poszthoz tartozó játékosok
              {
                  int take = 0;
                  switch (pos)
                  {
                      case Position.kapus:
                          take = 1;
                          break;
                      case Position.csatar:
                          take = forward;
                          break;
                      case Position.hatved:
                          take = defender;
                          break;
                      case Position.kozeppalyas:
                          take = midfielder;
                          break;
                      default:
                          throw new InvalidOperationException($"Ismeretlen pozíció: {pos}");
                  }
                  return new
                  {
                      Players = posPlayers.OrderByDescending(p => p.Value).Take(take).Select(p => p.Key).ToList(), // Vesszük a legjobb x-et, x a poszttól függ
                      Points = posPlayers.Sum(p => p.Value) // összeadjuk a pontjait
                  };
              });
            
            // A configparts-ban van benne, hogy egy adott posztra kik a legjobbak
            // Az ő pontjaiknak az összege adja meg a felállás összpontját
            this.Points = configParts.Sum(p => p.Points);
            this.Players = configParts.SelectMany(p => p.Players).ToList();
        }
    }
}
