﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace ConsoleApp7
{
    /// <summary>
    /// A fájlok feldolgozását megvalósító osztály
    /// </summary>
    public class FileProcessor
    {
        private readonly string gameFile;
        private readonly string playerFile;

        /// <summary>
        /// Létrehoz egy példányt a feldolgozáshoz
        /// </summary>
        /// <param name="gameFile">A feldolgozandó meccs file neve</param>
        /// <param name="playerFile">A feldolgozandó játékos file neve</param>
        public FileProcessor(string gameFile, string playerFile)
        {
            this.gameFile = gameFile;
            this.playerFile = playerFile;
        }

        /// <summary>
        /// Beolvassa a játékot a meccs file-ból
        /// </summary>
        /// <returns>Egy inicializált Game objektum, amely attól függően, hogy a játék hazai vagy válogatott meccs</returns>
        public Game GetGame()
        {
            // Kis beágyazott függvény, ami egy sort dolgoz fel a jegyzőkönyvből
            PlayerStat ProcessPlayerDataLine(string s)
            {
                var playerInfo = s.Split(new char[] { '(', ';', ')' }, StringSplitOptions.RemoveEmptyEntries);
                string name = playerInfo[0];
                bool yellowCard = false;
                bool redCard = false;
                int goals = 0;

                if (playerInfo.Length == 2)
                {
                    yellowCard = playerInfo[1] == "s";
                    redCard = playerInfo[1] == "k";
                }
                else
                {
                    goals = int.Parse(playerInfo[1]);
                    yellowCard = playerInfo[2] == "s";
                    redCard = playerInfo[2] == "k";
                }

                return new PlayerStat(name, goals, yellowCard, redCard);
            }

            var data = File.ReadAllLines(gameFile).ToList();
            var gameType = data[0]; // Első sor: a meccs típusa
            string team1Name = string.Empty;
            string team2Name = string.Empty;
            int team1Goals = 0;
            int team2Goals = 0;

            if (gameType == Game.GameTypes.Hazai) // Hazai meccs formátuma:
            {
                team1Name = data[1].Split(':')[0]; // Második sor: az eredmény
                team2Name = data[1].Split(':')[1];
                team1Goals = int.Parse(data[2].Split(':')[0]); // Harmadik sor: a csapatok nevei
                team2Goals = int.Parse(data[2].Split(':')[1]);
            }
            else if (gameType == Game.GameTypes.Valogatott) // Válogatott meccs esetén
            {
                team1Goals = int.Parse(data[1].Split(':')[0]); // Csak az eredmény van az első sorban
                team2Goals = int.Parse(data[1].Split(':')[1]);
            }
            else
            {
                // Egy kis best-practice; ha valami miatt van egy új meccstípus, legalább feldolgozáskor lássuk
                throw new InvalidOperationException($"Ismeretlen meccs típus: {gameType}");
            }

            data.RemoveRange(0, 3); // Az első három sort kiszedjük, ezekben nem jegyzőkönyv van még

            var playerStats = data.Select(x => ProcessPlayerDataLine(x)).ToList();
            if (gameType == Game.GameTypes.Hazai)
            {
                return new NationalGame(team1Name, team2Name, team1Goals, team2Goals, playerStats);
            }
            else
            {
                return new InternationalGame(team1Goals, team2Goals, playerStats);
            }
        }

        /// <summary>
        /// Beolvassa a játékosokat a játékos file-ból
        /// </summary>
        /// <returns>Egy lista a játékosok információval</returns>
        public List<Player> GetPlayers()
        {
            var playerInfo = File.ReadAllLines(playerFile);

            return (
                from p in playerInfo
                let s = p.Split(',') // vesszővel elválasztva a név, a csapat és a pozíció
                select new Player(s[0], s[1], s[2])
            ).ToList();
        }
    }
}
