﻿using System.Collections.Generic;
using System.Linq;

namespace ConsoleApp7
{
    /// <summary>
    /// Egy meccset reprezentáló alaposztály
    /// </summary>
    public abstract class Game
    {
        /// <summary>
        /// A meccstípusok leírásához
        /// </summary>
        public static class GameTypes
        {
            /// <summary>
            /// Hazai meccs
            /// </summary>
            public static string Hazai = "h";

            /// <summary>
            /// Válogatott meccs
            /// </summary>
            public static string Valogatott = "v";
        }

        /// <summary>
        /// Az egyik csapat góljai
        /// </summary>
        public int Team1Goals { get; }

        /// <summary>
        /// A másik csapat góljai
        /// </summary>
        public int Team2Goals { get; }

        /// <summary>
        /// A jegyzőkönyv elemei
        /// </summary>
        public List<PlayerStat> PlayerStats { get; }

        /// <summary>
        /// Igaz, ha a meccs döntetlen
        /// </summary>
        public bool IsTie => Team1Goals == Team2Goals;

        /// <summary>
        /// Létrehoz egy új Game objektumot
        /// </summary>
        /// <param name="team1Goals">Az egyik csapat góljai</param>
        /// <param name="team2Goals">A másik csapat góljai</param>
        /// <param name="playerStats">A jegyzőkönyv elemei</param>
        public Game(int team1Goals, int team2Goals, List<PlayerStat> playerStats)
        {
            this.Team1Goals = team1Goals;
            this.Team2Goals = team2Goals;
            this.PlayerStats = playerStats;
        }

        /// <summary>
        /// Kiszámolja egy adott játékos pontjait az adott meccsre vonatkozóan
        /// </summary>
        /// <param name="player">A játékos, akinek a pontjaira kíváncsiak vagyunk</param>
        /// <returns>Egy játékos pontjai az adott meccsre</returns>
        public int GetPlayerPoints(Player player)
        {
            var stat = PlayerStats.SingleOrDefault(p => p.Name == player.Name); // Ugye, nem kell keres függvény :)
            if (stat == null)
            {
                return 0;
            }
            var points = GetGamePoints(player) - GetCardPoints(stat); // Vesszük a meccsért járó pontokat, levontjuk a lapért járó pontokat
            if (player.Position == Position.kapus)
            {
                points = points + GetGoaliePoints(player); // ha kapus, akkor vesszük a kapuspontokat
            }
            else
            {
                points = points + GetGoalPoints(stat); // ha mezőnyjátékos, akkor vesszük a gólért járó pontokat
            }
            return points;
        }

        /// <summary>
        /// Kiszámolja, hogy hány pont jár a meccsért az adott játékosnak
        /// </summary>
        /// <param name="player">Az adott játékos</param>
        /// <returns>A meccs megnyeréséért/elvesztéséért járó pontok</returns>
        protected abstract int GetGamePoints(Player player);

        /// <summary>
        /// Kiszámolja, hogy a kapus mennyi "kapuspontot" szerzett a meccsen
        /// </summary>
        /// <param name="player">A kapus</param>
        /// <returns>A kapus pontjai</returns>
        protected abstract int GetGoaliePoints(Player player);

        /// <summary>
        /// A gólokért járó pontok száma
        /// </summary>
        /// <param name="stat">A jegyzőkönyv adott játékosra vonatkozó sora</param>
        /// <returns>A gólokért járó pontok</returns>
        protected abstract int GetGoalPoints(PlayerStat stat);

        private int GetCardPoints(PlayerStat stat)
        {
            if (stat.RedCard)
            {
                return 6;
            }
            if (stat.YellowCard)
            {
                return 3;
            }
            return 0;
        }
    }
}
