﻿using System;
using System.Collections.Generic;

namespace ConsoleApp7
{
    /// <summary>
    /// Válogatott meccs osztálya
    /// </summary>
    public class InternationalGame : Game
    {
        /// <summary>
        /// Megnyerte-e a csapat a meccset?
        /// </summary>
        public bool WonGame => Team1Goals > Team2Goals;

        /// <summary>
        /// Létrehoz egy új válogatott meccs objektumot
        /// </summary>
        /// <param name="team1Goals">Hazai gólok</param>
        /// <param name="team2Goals">Ellenfél góljai</param>
        /// <param name="playerStats">Jegyzőkönyv</param>
        public InternationalGame(int team1Goals, int team2Goals, List<PlayerStat> playerStats)
            : base(team1Goals, team2Goals, playerStats)
        {

        }

        /// <summary>
        /// Egy válogatott meccsért járó pontok
        /// </summary>
        /// <param name="player">Az adott játékos</param>
        /// <returns>A játékos adott meccsért járó pontjai</returns>
        protected override int GetGamePoints(Player player)
        {
            if (this.IsTie) // Döntetlenért 0 pont
                return 0;
            if (this.WonGame) // Győzelemért 6 pont
                return 6;
            return -3; //Vereségért 3 pont
        }

        /// <summary>
        /// A kapus pontjai az adott meccsért
        /// </summary>
        /// <param name="player">A kapus játékos</param>
        /// <returns>A kapus játékos pontjai</returns>
        protected override int GetGoaliePoints(Player player)
        {
            // Kis best-practice: Ezt csak kapussal lehessen meghívni
            if (player.Position != Position.kapus)
                throw new ArgumentException($"A megadott játékos ({player}) nem kapus");

            if (Team2Goals == 0) // Ha a kapus nem kapott gólt => Az ellenfél nem rúgott gólt => 12 pont
                return 12;
            if (Team2Goals > 1) // Ha a kapus több, mint 1 gólt kapott => Az ellenfél góljainak száma több, mint 1 => -6 pont
                return -6;
            return 0;
        }

        /// <summary>
        /// A gólokért járó pontokat adja vissza
        /// </summary>
        /// <param name="stat">Adott játékos pontjai</param>
        /// <returns>Adott játékos pontjai</returns>
        protected override int GetGoalPoints(PlayerStat stat)
        {
            // Megjegyzés: Itt is lehetne kis best-practice, hogy ne kapussal hívjuk meg
            // De a kapus nem lőhetne gólt? A jegyzőkönyv ezt most nem tudja, de később lehet, hogy fogja tudni
            // Most nem számít, a kapusnál a stat-ban a rúgott gólok úgyis 0
            return stat.Goals * 6;
        }
    }
}
