﻿using System;
using System.Collections.Generic;

namespace ConsoleApp7
{
    /// <summary>
    /// Klubbmeccseket leíró objektum
    /// </summary>
    public class NationalGame : Game
    {
        /// <summary>
        /// Az egyik csapat neve
        /// </summary>
        public string Team1Name { get; }

        /// <summary>
        /// A másik csapat neve
        /// </summary>
        public string Team2Name { get; }

        /// <summary>
        /// A győztes csapat nevét adja vissza
        /// </summary>
        public string WinnerTeam => Team1Goals > Team2Goals ? Team1Name : (Team2Goals > Team1Goals ? Team2Name : null);

        /// <summary>
        /// Visszaad egy klubmeccset leíró objektumot
        /// </summary>
        /// <param name="team1Name">Az egyik csapat neve</param>
        /// <param name="team2Name">A másik csapat neve</param>
        /// <param name="team1Goals">Az egyik csapat góljai</param>
        /// <param name="team2Goals">A másik csapat góljai</param>
        /// <param name="playerStats">A jegyzőkönyv elemei</param>
        public NationalGame(string team1Name, string team2Name, int team1Goals, int team2Goals, List<PlayerStat> playerStats)
            : base(team1Goals, team2Goals, playerStats)
        {
            this.Team1Name = team1Name;
            this.Team2Name = team2Name;
        }

        /// <summary>
        /// Egy válogatott meccsért járó pontok
        /// </summary>
        /// <param name="player">Az adott játékos</param>
        /// <returns>A játékos adott meccsért járó pontjai</returns>
        protected override int GetGamePoints(Player player)
        {
            if (this.IsTie) // Döntetlen esetén 0 pont
                return 0;
            if (player.Team == this.WinnerTeam) // Ha győzött, akkor 3 pont
                return 3;
            return -3; // Egyébként -3 pont
        }

        /// <summary>
        /// A kapus pontjai az adott meccsért
        /// </summary>
        /// <param name="player">A kapus játékos</param>
        /// <returns>A kapus játékos pontjai</returns>
        protected override int GetGoaliePoints(Player player)
        {
            // Kis best-practice: Ezt csak kapussal lehessen meghívni
            // Még szebb lenne, ha ezt az egy ellenőrzést az ősosztályba tenném
            if (player.Position != Position.kapus)
                throw new ArgumentException($"A megadott játékos ({player}) nem kapus");

            if (player.Team == this.Team1Name && this.Team2Goals == 0) // Az ellenfél nem rúg gólt
                return 6;
            if (player.Team == this.Team2Name && this.Team1Goals == 0)
                return 6;
            if (player.Team == this.Team1Name && this.Team2Goals > 1) // Az ellenfél több, mint 1 gólt rúg
                return -6;
            if (player.Team == this.Team2Name && this.Team1Goals > 1)
                return -6;
            return 0;
        }

        /// <summary>
        /// A gólokért járó pontokat adja vissza
        /// </summary>
        /// <param name="stat">Adott játékos pontjai</param>
        /// <returns>Adott játékos pontjai</returns>
        protected override int GetGoalPoints(PlayerStat stat)
        {
            // Megjegyzés: Itt is lehetne kis best-practice, hogy ne kapussal hívjuk meg
            // De a kapus nem lőhetne gólt? A jegyzőkönyv ezt most nem tudja, de később lehet, hogy fogja tudni
            // Most nem számít, a kapusnál a stat-ban a rúgott gólok úgyis 0
            return stat.Goals * 3;
        }
    }
}
