﻿using System;

namespace ConsoleApp7
{
    /// <summary>
    /// Egy játékost leíró objektum
    /// </summary>
    public class Player
    {
        /// <summary>
        /// Létrehoz egy játékos objektumot
        /// </summary>
        /// <param name="name">A játékos neve</param>
        /// <param name="team">A játékos csapata</param>
        /// <param name="position">A játékos posztja, karakterláncként</param>
        public Player(string name, string team, string position)
        {
            this.Name = name;
            this.Team = team;
            this.Position = (Position)Enum.Parse(typeof(Position), position); // A könnyebb kezelhetőségért és kisebb hibalehetőségért a string-et átalakítjuk enummá
        }

        /// <summary>
        /// A játékos neve
        /// </summary>
        public string Name { get; }

        /// <summary>
        /// A játékos csapata
        /// </summary>
        public string Team { get; }

        /// <summary>
        /// A játékos posztja
        /// </summary>
        public Position Position { get; }

        /// <summary>
        /// Visszaadja a játékos adatait karakterláncba összefűzve
        /// </summary>
        /// <returns>A játékos adatai karakterláncként</returns>
        /// <remarks>Csak hibakeresési célokra!!</remarks>
        public override string ToString()
        {
            return $"{this.Name},{this.Team},{this.Position}";
        }

        /// <summary>
        /// Összehasonlít két játékos objektumot és megmondja, hogy egyenlőek-e
        /// </summary>
        /// <param name="obj">A másik objektum</param>
        /// <returns>Igaz, ha a két játékos ugyanaz az objektum, vagy ugyanaz a nevük, csapatuk és posztjuk</returns>
        /// <remarks>Listában kereséshez, szótárban tároláshoz szükséges lehet</remarks>
        public override bool Equals(object obj)
        {
            if (object.ReferenceEquals(this,obj))
                return true;

            var other = (Player)obj;
            return this.Name == other.Name && this.Team == other.Team && this.Position == other.Position;
        }

        /// <summary>
        /// Visszaadja egy játékos objektum hashcode-ját
        /// </summary>
        /// <returns>Az objektum hashcode-ja</returns>
        /// <remarks>Listában kereséshez, szótárban tároláshoz szükséges lehet</remarks>
        public override int GetHashCode()
        {
            return this.Name.GetHashCode() ^ this.Team.GetHashCode() ^ this.Position.GetHashCode();
        }
    }
}
