﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConsoleApp7
{
    class Program
    {
        // Előző eredmények beolvasása
        static Dictionary<Player, int> GetPreviousResults(string fileName)
        {
            if (!File.Exists(fileName))
            {
                return new Dictionary<Player, int>();
            }
            return (
                from p in File.ReadAllLines(fileName)
                let s = p.Split(';')
                select new { Player = new Player(s[0], s[1], s[2]), Points = int.Parse(s[3]) })
             .ToDictionary(x => x.Player, x => x.Points);

        }

        // Eredmények mentése
        static void SaveResults(string fileName, Dictionary<Player, int> gameStats)
        {
            var content = gameStats.Select(gs => $"{gs.Key.Name};{gs.Key.Team};{gs.Key.Position.ToString()};{gs.Value}").ToList();
            File.WriteAllLines(fileName, content);
        }
        static void Main(string[] args)
        {
            // Felderítjük a fájlokat
            var matchFiles = Directory.GetFiles(Directory.GetCurrentDirectory(), "meccs*.txt");
            var playerFiles = Directory.GetFiles(Directory.GetCurrentDirectory(), "jatekos*.txt");

            // Beolvassuk az előző pontokat (ha voltak)
            var totalPoints = GetPreviousResults("eredmeny.txt");

            // Páronként végigmegyünk a játékos - meccs fájlokon
            foreach (var filePair in matchFiles.Zip(playerFiles, (matchFile, playerFile) => new { matchFile, playerFile }))
            {
                var fileProcessor = new FileProcessor(filePair.matchFile, filePair.playerFile);
                var game = fileProcessor.GetGame(); // beolvassuk a játékot
                var players = fileProcessor.GetPlayers(); // beolvassuk a játékosokat
                var playerMatchPoints = players.ToDictionary(p => p, p => game.GetPlayerPoints(p)); // Kiszámoljuk a pontokat
                foreach (var playerMatchPoint in playerMatchPoints)
                {
                    if (totalPoints.ContainsKey(playerMatchPoint.Key)) // Ha volt már ennek a játékosnak pontja, hozzáadjuk az előzőhöz
                    {
                        totalPoints[playerMatchPoint.Key] += playerMatchPoint.Value;
                    }
                    else // Ha nem volt még pontja, akkor létrehozunk egy bejegyzést neki
                    {
                        totalPoints[playerMatchPoint.Key] = playerMatchPoint.Value;
                    }
                }
            }

            // Eredmények mentése
            SaveResults("eredmeny.txt", totalPoints);

            // Létrehozzuk az összes lehetséges felállást
            List<Config> configs = new List<Config>
            {
                new Config(3, 4, 3, totalPoints),
                new Config(3, 5, 2, totalPoints),
                new Config(4, 4, 2, totalPoints),
                new Config(4, 3, 3, totalPoints),
            };

            // Megkeressük a legjobbat
            // Rendezzük és vesszük az elsőt
            var bestConfig = configs.OrderByDescending(c => c.Points).First();

            // Kiírjuk a játékosokat
            var content = bestConfig.Players.Select(p => $"{p.Name},{p.Position}").ToList(); 
            File.WriteAllLines("lista.txt", content);

            Console.ReadLine();
        }
    }
}
